/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <sys/printk.h>
#include <logging/log.h>

#define PRIORITY 7
#define STACKSIZE 1024

K_SEM_DEFINE(custom_sem, 0, 1);
LOG_MODULE_REGISTER(sem_timing, 3);

#define PIN_GPIO  (27UL)
struct k_timer my_timer;
/*
	-Set up a timer and trigger it every 4ms and toggles a GPIO
        and attach a logic analyzer to see if it triggers that often
		-Ok, it did
	-Create a new thread and get that up and running
		-Ok, it works
	-Each time the timer triggers, give a semaphore and then get it in the new thread and
	attach a logic analyzer wire in the thread as well and check if it triggers every 4ms
		-That worked perfectly fine
	-Add a LOG_INF() and see if it still works

*/
bool flipper = true;

void timer_event(struct k_timer *timer_id){
	k_sem_give(&custom_sem);
}

void custom_thread(void)
{
	while(true){
		k_sem_take(&custom_sem, K_FOREVER);
		LOG_INF("HELLO FROM CUSTOM THREAD");
		if(flipper){
			NRF_P0_S->OUTSET = (1UL << PIN_GPIO); 
			flipper = false;
		}else{
			NRF_P0_S->OUTCLR = (1UL << PIN_GPIO);  
			flipper = true;
		}
	}
}

void main(void)
{
    printk("Hello World! %s\n", CONFIG_BOARD);
    //Timer setup
    NRF_P0_S->PIN_CNF[PIN_GPIO] = (GPIO_PIN_CNF_DIR_Output << GPIO_PIN_CNF_DIR_Pos) |
				(GPIO_PIN_CNF_DRIVE_S0S1 << GPIO_PIN_CNF_DRIVE_Pos) |
				(GPIO_PIN_CNF_INPUT_Connect << GPIO_PIN_CNF_INPUT_Pos) |
				(GPIO_PIN_CNF_PULL_Disabled << GPIO_PIN_CNF_PULL_Pos) |
				(GPIO_PIN_CNF_SENSE_Disabled << GPIO_PIN_CNF_SENSE_Pos);

    k_timer_init(&my_timer, timer_event, NULL);
    k_timer_start(&my_timer, K_MSEC(4), K_MSEC(4));
}


K_THREAD_DEFINE(custom_thread_id, STACKSIZE, custom_thread, NULL, NULL, NULL,
		PRIORITY, 0, 0);